from .papers import author_papers_jump, citations_jump, references_jump
from .search import search_by_title_jump
from .detail import paper_detail_jump
from .favorites import add_favorite_paper_jump, show_favorite_paper_jump, rm_favorite_paper_jump
from .favorites import add_favorite_keywords_jump, show_favorite_keywords_jump, rm_favorite_keywords_jump
